/*
 *  Main detection routines
 *  Copyright (c) by Jaroslav Kysela <perex@jcu.cz>
 */

#define ULTRA_MAIN_OBJECT_FILE
#include "driver.h"
#include <stdarg.h>

#define DETECT_VERSION		"1.00a"
#define DETECT_BUFFER_SIZE	(32 * 1024)

extern int ultra_detect_interwave( ultra_card_t *card, unsigned short port );

static struct ULTRA_STRU_DETECT_ISA {
  unsigned short ports[ 8 ];	/* possible ports, 0 = terminator */
  char *name;			/* card name */
  int (*probe)( ultra_card_t *card, unsigned short port );
} ultra_detect_isa[] = {
 {
   { 0x210, 0x220, 0x230, 0x240, 0x250, 0x260, 0 },
   "AMD InterWave",
   ultra_detect_interwave
 },
};

#define ULTRA_DETECT_ISA_COUNT (sizeof(ultra_detect_isa)/sizeof(struct ULTRA_STRU_DETECT_ISA))

static void ultra_detect_use_inc( void )
{
  MOD_INC_USE_COUNT;
}

static void ultra_detect_use_dec( void )
{
  MOD_DEC_USE_COUNT;
}

static int ultra_detect_init( ultra_info_buffer_t *buffer )
{
  struct ULTRA_STRU_DETECT_ISA *isa;
  unsigned short *ports;
  int idx, ok, cardidx;
  ultra_card_t *cards[ ULTRA_CARDS ];	/* detected cards */

  buffer -> curr = buffer -> buffer;
  *buffer -> buffer = 0;
  ultra_iprintf( buffer, "Version: " DETECT_VERSION "\n" );
  if ( ultra_cards_count > 0 ) {
    return -EBUSY;
  }
  MEMSET( &cards, 0, sizeof( cards ) );
  cardidx = 0;
  for ( idx = 0; idx < ULTRA_CARDS; idx++ ) {
    cards[ idx ] = ultra_card_new( NULL, NULL, ultra_detect_use_inc, ultra_detect_use_dec );
    if ( !cards[ idx ] ) {
      for ( idx = 0; idx < ULTRA_CARDS; idx++ ) {
        if ( cards[ idx ] ) ultra_card_free( cards[ idx ] );
      }
      return -ENOMEM;
    }
  }
  for ( idx = 0; cardidx < ULTRA_CARDS && idx < ULTRA_DETECT_ISA_COUNT; idx++ ) {
    isa = &ultra_detect_isa[ idx ];
    ok = 0;
    for ( ports = isa -> ports; cardidx < ULTRA_CARDS && *ports; ports++ ) {
      if ( !isa -> probe( cards[ cardidx ], *ports ) ) {
        ultra_iprintf( buffer, "ISA#%s#0x%x\n", isa -> name, *ports );
        cardidx++;
      }
    }
  }
  for ( idx = 0; idx < ULTRA_CARDS; idx++ ) {
    if ( cards[ idx ] ) ultra_card_free( cards[ idx ] );
  }
  return 0;
}

static long ultra_detect_read( struct file *file, char *buffer, long count )
{
  ultra_info_buffer_t *buf;
  int size, size1, err;
  
  buf = (ultra_info_buffer_t *)file -> private_data;
  if ( !buf ) return -EIO;
  if ( !buf -> size ) {
    err = ultra_detect_init( buf );
    if ( err < 0 ) return err;
  }
  if ( file -> f_pos >= buf -> size ) return 0;
  size = buf -> size < count ? buf -> size : count;
  size1 = buf -> size - file -> f_pos;
  if ( size1 < size ) size = size1;
  MEMCPY_TOFS( buffer, buf -> buffer + file -> f_pos, size );
  file -> f_pos += size;
  return size;
}

static int ultra_detect_open( unsigned short minor, struct file *file )
{
  ultra_info_buffer_t *buf;
  
  buf = (ultra_info_buffer_t *)ultra_malloc( sizeof( ultra_info_buffer_t ) );
  if ( !buf ) return -ENOMEM;
  MEMSET( buf, 0, sizeof( ultra_info_buffer_t ) ); 
  buf -> buffer = (char *)VMALLOC( DETECT_BUFFER_SIZE );
  if ( !buf -> buffer ) {
    ultra_free( buf, sizeof( ultra_info_buffer_t ) );
    return -ENOMEM;
  }
  file -> private_data = buf;
  MOD_INC_USE_COUNT;
  return 0;
}

static int ultra_detect_release( unsigned short minor, struct file *file )
{
  ultra_info_buffer_t *buf;
  
  buf = (ultra_info_buffer_t *)file -> private_data;
  file -> private_data = NULL;
  if ( buf ) {
    if ( buf -> buffer ) VFREE( buf -> buffer );
    ultra_free( buf, sizeof( ultra_info_buffer_t ) );
  }  
  MOD_DEC_USE_COUNT;
  return 0;
}

/*
 *
 */

static int ultra_detect_unregister( void );

static struct ultra_stru_register_minor ultra_detect_reg = {
  "ultradetect",

  0,					/* don't verify soundcard # */

  { ULTRA_MINOR_UDETECT, -1 },
  { -1 },
  
  ultra_detect_unregister,		/* unregister */

  NULL,					/* lseek */
  ultra_detect_read,			/* read */
  NULL,					/* write */
  ultra_detect_open,			/* open */
  ultra_detect_release,			/* release */
  NULL,					/* poll or select */
  NULL,					/* ioctl */
  NULL					/* mmap */
};

static int ultra_detect_unregister( void )
{
  ultra_unregister_minor( &ultra_detect_reg );
  return 0;
}

int init_module( void )
{
  int err;

  if ( ( err = ultra_register_minor( &ultra_detect_reg ) ) < 0 )
    return err;
  return 0;
}

void cleanup_module( void )
{
  ultra_detect_unregister();
}
